import argparse
import os
import sys

parent_dir = os.path.join(os.getcwd(),os.pardir)
sys.path.append(parent_dir)
# Please have both XigtifiedToolbox and mom from aggregation/mom on your PYTHONPATH.
from mom import MainClasses
from . import XigtReader
from . import report_problems


parser = argparse.ArgumentParser(description="Loads a settings file and run a set of tests.")

parser.add_argument("-s", "--settings", help="The name of the configuration file.")

parser.add_argument("-v", "--verbose", help="Verbose output", action='store_true')

args = parser.parse_args()

def main():
    if len(sys.argv) < 2:
        print('AGG-MOM usage: mom -s config_file [-v (verbose)]')
        sys.exit(1)
    settings_file = args.settings
    print ("Loading Configuration File",settings_file)
    settings = MainClasses.Settings(settings_file)
    xigt_reader = XigtReader.Xigt_Reader(settings, verbose=args.verbose)
    xigt_reader.process_igts(settings.data_file)
    xigt_reader.print_skipped(settings.out_dir + '/skipped_items.txt')
    pr = report_problems.ProblemReport()
    pr.report_inconsistencies(settings.toolbox, settings.reftag, settings.lang_line_tag, settings.toolbox_tags,
                              xigt_reader.skipped_words,xigt_reader.full_xc)

if __name__ == '__main__':
    main()


from mom.XigtifiedToolbox.util import fetch_igts

class ProblemReport:
    def report_inconsistencies(self, toolbox_file, reftag, lang_line_tag, tags, problems, xc):
        with open(toolbox_file, 'r') as f:
            toolbox_lines = f.readlines()
        nonemtpy_toolbox_lines = [l for l in toolbox_lines if l.strip()]
        toolbox_igts = self.parse_toolbox(nonemtpy_toolbox_lines, reftag, tags)
        output_lines = []
        for igt_id in problems:
            tb_id = igt_id
            if len(problems[igt_id]) > 0:
                toolbox_entry = toolbox_igts.get(igt_id)
                if not toolbox_entry:
                    tb_id = igt_id[3:]
                    toolbox_entry = toolbox_igts.get(tb_id) #maybe 'igt' was prepended by xigt
                    if not toolbox_entry:
                        fetch_igts(xc,[igt_id])
                        raise Exception('Toolbox entry with reference {0} not found'.format(igt_id))
                text = toolbox_entry[lang_line_tag]
                w_ids = []
                comments = []
                for p in problems[igt_id]:
                    w_id,word = p.split('\t')
                    w_ids.append(w_id)
                    comment = problems[igt_id][p]
                    comments.append(comment + ' in word #{0} ({1})\n'.format(w_id,word))
                    additional_comment = self.create_additional_comment(comment,toolbox_entry,w_id)
                    comments.append(additional_comment)
                self.construct_output_entry(toolbox_entry,w_ids,output_lines,toolbox_lines,reftag,tb_id,comments)
        with open('problems.txt', 'w') as f:
            for ln in output_lines:
                if ln:
                    f.write(ln)

    def create_additional_comment(self,comment,toolbox_entry,w_id):
        if comment.startswith('No gloss'):
            return "COMMENT: Probably a missing or extra space somewhere in the IGT.\n"

    def construct_output_entry(self,toolbox_entry,word_ids,output_lines,toolbox_lines,reftag,igt_id,comments):
        relevant_lines = self.find_toolbox_lines(toolbox_lines,reftag,igt_id)
        output_lines.append("****PROBLEMATIC ENTRY ID: {0}****\n".format(igt_id))
        output_lines.append("PROBLEM CODE(s) AS DETECTED BY XIGT:\n")
        output_lines.extend(comments)
        output_lines.append('\n')
        output_lines.extend(relevant_lines)
        output_lines.append('\n')

    # def detect_misalignment(self,toolbox_igt):
    #     misalignments = []
    #     first_line = toolbox_igt[0]
    #     for i,c in enumerate(first_line):
    #         for ln in toolbox_igt[1:]:
    #             if not (c == ' ' and ln[i] == ' ') or not (c != ' ' and ln[i] != ' '):
    #                 misalignments.append(i)
    #     return misalignments




    def parse_toolbox(self, toolbox_lines, reftag, tags):
        igts = {}
        i = 0
        while i < len(toolbox_lines)-1:
            while not toolbox_lines[i].startswith(reftag) and i < len(toolbox_lines)-1:
                i += 1
            cur_id = toolbox_lines[i].strip()[5:] # strip the newline and the \\ref tag.
            igts[cur_id] = {}
            i += 1
            prev_tag = None
            while i < len(toolbox_lines) - 1 and not toolbox_lines[i].startswith(reftag):
                tag = toolbox_lines[i].split(' ')[0]
                if not tag.startswith('\\'):
                    if not prev_tag:
                        raise Exception('Why was prev_tag not been initialized?')
                    if prev_tag in tags:
                        igts[cur_id][prev_tag].append(toolbox_lines[i])
                else:
                    prev_tag = tag
                    if tag in tags:
                        if not tag in igts[cur_id]:
                            igts[cur_id][tag] = []
                        igts[cur_id][tag].append(toolbox_lines[i])
                i += 1
        return igts

    def find_toolbox_lines(self, toolbox_lines, reftag, igt_id):
        relevant_lines = []
        i=0
        while i < len(toolbox_lines) - 1 and not toolbox_lines[i].startswith(reftag + ' ' + igt_id):
            i += 1
        while i < len(toolbox_lines) - 1 and not toolbox_lines[i+1].startswith(reftag):
            relevant_lines.append(toolbox_lines[i])
            i += 1
        return relevant_lines



import sys

toolbox_file = sys.argv[1]
identifier = sys.argv[2]


pos_tags = set()

with open(toolbox_file,'r') as tf:
    lines = tf.readlines()

for ln in lines:
    if ln.startswith('\\' + identifier):
        tags = ln.split()
        for t in tags:
            pos_tags.add(t)

with open(identifier + '-tags.txt', 'w') as of:
    for t in sorted(list(pos_tags)):
        of.write(t+'\n')


'''
Usage: python3 collect_tags_xigt.py xigt_file.xml pos_tier_id gloss_tier_ids output_dir

E.g.:

python3 ../sample_data/eng.xml pos g ../sample_data/

This will simply collect all grams from the tiers that correspond to the supplied tier ids (e.g. pos and g) and output
two files: pos-tags.txt and glosses.txt. These files can then be copied somewhere safe and then the path to them
should be given to MOM in the config file: pos-tags.txt should be divided into verb tags and noun/pronoun tags (two
separate files). glosses.txt should be filtered to only contain glosses (as opposed to stem translations).

Note: Again, you need to *MANUALLY* delete everything you don't want in those files. But it isn't too bad: it usually
only takes a few minutes per dataset. True grams tend to gravitate to the top generally and to the top of each letter
of the alphabet.
'''

import sys
import os
from xigt.codecs import xigtxml
from .FeatDict import FeatureDictionary

class TagCollector():
    def __init__(self,xigt_file,pos_tier_id,gloss_tier_ids):
        self.xigt_file = xigt_file
        self.pos_tier_id = pos_tier_id
        self.gloss_tier_ids = gloss_tier_ids

    def collect_tags(self):
        self.pos_tags = set()
        self.glosses = set()
        with open(self.xigt_file, encoding='utf-8') as file:
            xc = xigtxml.load(file)

        # get unique POS tags and glosses
        for igt in xc:
            pos_tier = igt.get(self.pos_tier_id)
            if pos_tier:
                for item in pos_tier:
                    if item.value():
                        self.pos_tags.add(item.value())
            for g_id in self.gloss_tier_ids:
                gl_tier = igt.get(g_id)
                if gl_tier:
                    for item in gl_tier:
                        if item.value():
                            self.glosses.add(item.value())


    def write_tags_to_file(self, filename, tags):
        with open(filename, 'w', encoding='utf8') as of:
            for t in sorted(list(tags)):
                of.write(t+'\n')


    def check_for_standard_glosses(self):
        fd = FeatureDictionary()
        self.unknown_features = set()
        updated_glosses = set()

        # check for standard glosses and remove nonstandard ones from the glosses set,
        # putting them in the unknown set
        for g in self.glosses:
            for gram in g.strip('-=').split('.'):
                if (not gram.lower() in fd.all_keys):
                    self.unknown_features.add(gram)
                else:
                    updated_glosses.add(gram)

        self.glosses = updated_glosses


        # print('The following glosses could not be mapped to anything in the Feature Dictionary:')
        # print(';'.join(sorted(list(unknown_features))))
        # print('If there are not just stems but also glosses for morphosyntactic features among them, '
        #       'consider adding the lower case version of them to the appropriate part '
        #       'of the Feature Dictionary. Look in unknown_features.txt')

def main():
    tc = TagCollector(sys.argv[1],sys.argv[2],sys.argv[3].split(','))
    tc.collect_tags()
    tc.check_for_standard_glosses()
    tc.write_tags_to_file(os.path.join(sys.argv[4], 'pos-tags.txt'),tc.pos_tags)
    tc.write_tags_to_file(os.path.join(sys.argv[4], 'glosses.txt'),tc.glosses)
    tc.write_tags_to_file(os.path.join(sys.argv[4], 'unknown-features.txt'),tc.unknown_features)


if __name__ == '__main__':
    main()



import sys
from xigt.codecs import xigtxml

def fetch_igts(xc,igt_ids):
    subset = []
    for id in igt_ids:
        subset.append(xc[id])
    subset_xc = xigtxml.XigtCorpus(igts=subset)
    xigtxml.dump('debug.xml',subset_xc)

def split_igts(xc, percent):
    subset = []
    #for i,igt in enumerate(xc):
    #    if i > len(xc)*percent:
    #        break
    #    subset.append(igt)
    subset = xc[:int(len(xc)*percent)]
    subset_xc = xigtxml.XigtCorpus(igts=subset)
    xigtxml.dump('debug.xml',subset_xc)


def unique_id(toolbox_file, reftag):
    map = {}
    new_toolbox_lines = []
    with open(toolbox_file,'r') as f:
        lines = f.readlines()
    for ln in [l.strip() for l in lines]:
        if ln.startswith(reftag):
            if not ln in map:
                map[ln] = []
            n = len(map[ln])
            if n == 0:
                map[ln].append(ln)
                new_toolbox_lines.append(ln)
            else:
                newln = ln + '-' + str(n)
                map[ln].append(newln)
                new_toolbox_lines.append(newln)
        else:
            new_toolbox_lines.append(ln)
    with open('uniq.txt','w') as f:
        for ln in new_toolbox_lines:
            f.write(ln + '\n')
    with open('map.txt','w') as f:
        f.write('The following non-unique \\ref entries were replaced by unique ids:\n')
        for k in map:
            if len(map[k]) > 1:
                for v in map[k]:
                    f.write(k + ':\t' + v + '\n')

def all_tags_for_word(word, xc):
    attested_tags = set()
    count = 0
    for igt in xc:
        pos_tier = igt.get('pos') if igt.get('pos') else igt.get('w-pos')
        if not pos_tier:
            pos_tier = igt.get('gw-pos')
        aligned_tier = pos_tier.alignment
        for item in pos_tier:
            relevant_word = igt.get_item(item.alignment)
            if relevant_word.value() == word:
                count += 1
                attested_tags.add(item.text)
    with open('attested_tags.txt', 'w') as f:
        f.write(word + ': ' + 'count=' + str(count) + '\n')
        for tag in attested_tags:
            f.write(tag + '\n')







if __name__ == '__main__':
    print('Uncomment/add whatever you want it to do.')
    #unique_id(sys.argv[1], sys.argv[2])
    #with open(sys.argv[1], 'r') as f:
    #    xc = xigtxml.load(f)
    #split_igts(xc,0.25)
    #all_tags_for_word('akka',xc)


import os
import re
import sys

import editdistance
from xigt import xigtpath as xp
from xigt.codecs import xigtxml
parent_dir = os.path.join(os.getcwd(),os.pardir)
sys.path.append(parent_dir)
from . import MorphTypeGuesser, FeatDict

ROOTS = {'root','stem','bound_root','bound root'}

class Xigt_Reader:
    def __init__(self, settings, verbose=False):
        self.settings = settings
        self.verb_tags = settings.verb_tags
        self.noun_tags = settings.noun_tags
        self.patterns = settings.patterns
        self.use_gloss_for_patterns = settings.use_gloss_to_precluster
        self.hyphens = settings.hyphens
        self.full_xc = None
        self.xc = []
        self.igt2items = {}
        self.items = {}
        self.items['verb'] = []
        self.items['noun'] = []
        self.ignore_igt = settings.ignore_igt
        self.ignore_chars = settings.ignore_chars
        self.verbose = verbose
        self.allowed_diff = settings.allowed_diff
        self.allomorphs = settings.allomorphs
        self.ungrammatical = settings.ungrammatical
        self.mtg = MorphTypeGuesser.MorphTypeGuesser(settings.known_glosses_file)
        self.lexitem_classes = settings.lexitem_classes
        self.assume_morph_boundaries = settings.assume_morph_boundaries
        self.all_bare = settings.all_bare
        self.skipped_igt = {} # id : reason
        self.skipped_words = {}

    def process_igts(self, file_name):
        print("Using file " + file_name)
        with open(file_name, encoding='utf-8') as file:
            xc = xigtxml.load(file)
            self.full_xc = xc
        print ("Processing Xigt IGTs...")
        (count,skipped, pos_words) = self.construct_pos_items(xc,self.verb_tags,self.noun_tags)
        print("Skipped {0} out of {1} words due to morpheme-to-gloss alignment issues. "
              "Look in output/skipped_items.txt".format(skipped,count))
        for pos in pos_words:
            self.items[pos] = sorted(pos_words[pos], key=lambda x: x.tokenized_text)
        self.igt2items = map_igt2words(self)

    def print_skipped(self, path):
        with open(path, 'w', encoding='utf-8') as f:
            f.write('SKIPPED WORDS:\n')
            for igt_id in self.skipped_words:
                if len(self.skipped_words[igt_id]) > 0:
                    f.write(igt_id + '\t')
                    for wd_id in self.skipped_words[igt_id]:
                        f.write(wd_id + '\t' + self.skipped_words[igt_id][wd_id] + '\n')
            f.write('SKIPPED IGT:\n')
            for igt_id in self.skipped_igt:
                f.write(igt_id + '\t' + self.skipped_igt[igt_id] + '\n')

    def collect_relevant_word_ids(self, xc, verb_tags,noun_tags):
        igt_to_word_ids = {}
        igt_to_word_ids['verb'] = {}
        igt_to_word_ids['noun'] = {}
        igt_count = 0
        igt_skipped = 0
        empty = 0
        for igt in xc:
            igt_count += 1
            igt_to_word_ids['verb'][igt.id] = []
            igt_to_word_ids['noun'][igt.id] = []
            self.skipped_words[igt.id] = {} # position : reason
            if not igt.get('p'):
                empty += 1
                self.skipped_igt[igt.id] = 'empty'
                if self.verbose:
                    print("Skipping empty IGT {0}".format(igt.id))
                continue
            if igt.get('p')[0].value() in self.ignore_igt:
                empty += 1
                self.skipped_igt[igt.id] = 'empty'
                if self.verbose:
                    print("Skipping empty IGT {0}".format(igt.id))
                continue
            else:
                self.normalize_text(igt)
                self.xc.append(igt)
            # extract all words from the corpus which have the right pos tag and seem part of natural dialog
            pos_tier = igt.get('pos') if igt.get('pos') else igt.get('w-pos')
            if not pos_tier:
                pos_tier = igt.get('m-pos')
            if not pos_tier:
                pos_tier = igt.get('gw-pos')
            if not pos_tier:
                pos_tier = igt.get('x')
            if not pos_tier:
                self.skipped_igt[igt.id] = 'no POS tier'
                if self.verbose:
                    print("No POS tier in IGT #{0} ({1}) skipping it".format(igt_count,igt.id))
                igt_skipped +=1
                continue
            else:
                aligned_tier = pos_tier.alignment
                if aligned_tier == 'w':
                    for item in pos_tier:
                        if item.text in verb_tags:
                            igt_to_word_ids['verb'][igt.id].append(item.alignment)
                        elif item.text in noun_tags:
                            igt_to_word_ids['noun'][igt.id].append(item.alignment)
                # KPH added to get alignment when gw-pos alignes to gw
                elif aligned_tier == 'gw':
                    for item in pos_tier:
                        if item.text in verb_tags:
                            aligned_word = igt.get('gw').get(item.alignment).alignment
                            igt_to_word_ids['verb'][igt.id].append(aligned_word)
                        elif item.text in noun_tags:
                            aligned_word = igt.get('gw').get(item.alignment).alignment
                            igt_to_word_ids['noun'][igt.id].append(aligned_word)
                elif aligned_tier == 'm':
                    tagged_words = {}# KPH we want to make sure that the same word isn't added multiple times, so we'll collect the words each tag points to and add them to igt_to_words at the end
                    for item in pos_tier:
                        if item.text in verb_tags or item.text in noun_tags:
                            relevant_word_id = igt.get_item(item.alignment).alignment
                            if not relevant_word_id:
                                relevant_word_id = igt.get_item(item.alignment).segmentation
                            if not relevant_word_id:
                                if self.verbose:
                                    print('Unexpected alignment structure: pos tier not '
                                                'aligned to either morphemes or words. '
                                      'Skipping IGT #{0} ({1})'.format(igt_count,igt.id))
                                igt_skipped += 1
                                self.skipped_igt[igt.id] = 'misalignment'
                                continue
                            if item.text in verb_tags:
                                if relevant_word_id in tagged_words:
                                    if 'verb' in tagged_words[relevant_word_id]:
                                        tagged_words[relevant_word_id]['verb'].append(item.alignment)
                                    else:
                                        tagged_words[relevant_word_id]['verb'] = [item.alignment]
                                else:
                                    tagged_words[relevant_word_id] = {'verb': [item.alignment]}
                                # if not relevant_word_id in igt_to_word_ids['verb'][igt.id]:
                                #     igt_to_word_ids['verb'][igt.id].append(relevant_word_id)
                            if item.text in noun_tags:
                                if relevant_word_id in tagged_words:
                                    if 'noun' in tagged_words[relevant_word_id]:
                                        tagged_words[relevant_word_id]['noun'].append(item.alignment)
                                    else:
                                        tagged_words[relevant_word_id]['noun'] = [item.alignment]
                                else:
                                    tagged_words[relevant_word_id] = {'noun': [item.alignment]}
                                    # if not relevant_word_id in igt_to_word_ids['noun'][igt.id]:
                                    #     igt_to_word_ids['noun'][igt.id].append(relevant_word_id)
                    for word_id in tagged_words:
                        if 'verb' in tagged_words[word_id] and 'noun' not in tagged_words[word_id]:
                            igt_to_word_ids['verb'][igt.id].append(word_id)
                        elif 'noun' in tagged_words[word_id] and 'verb' not in tagged_words[word_id]:
                            igt_to_word_ids['noun'][igt.id].append(word_id)
                        elif 'verb' in tagged_words[word_id] and 'noun' in tagged_words[word_id]:
                            # KPH if either morpheme is the root, we want to go with that pos. otherwise, we'll default to verb
                            roots = []
                            for pos in ['verb', 'noun']:
                                for morph_id in tagged_words[word_id][pos]:
                                    if self.assume_morph_boundaries:
                                        type = self.mtg.infer_morph_type_with_boundary(igt.get('m').get(morph_id).value())
                                    else:
                                        gloss_id = ''
                                        for g in igt.get('g'):
                                            if g.alignment == morph_id:
                                                gloss_id = g.id
                                        morpheme_glosses, root_indices = self.mtg.infer_morph_type_without_boundary([(igt.get('m').get(morph_id), [igt.get('g').get(gloss_id)])])
                                        if root_indices != []:
                                            type = 'root'
                                        else:
                                            type = ''
                                    if type == 'root':
                                        roots.append(pos)
                            if 'verb' in roots and 'noun' not in roots:
                                igt_to_word_ids['verb'][igt.id].append(word_id)
                            elif 'noun' in roots and 'verb' not in roots:
                                igt_to_word_ids['noun'][igt.id].append(word_id)
                            else:
                                igt_to_word_ids['verb'][igt.id].append(word_id)

                else:
                    if self.verbose:
                        print('Unexpected alignment structure: pos tier not '
                                    'aligned to either morphemes or words. '
                          'Skipping IGT #{0} ({1})'.format(igt_count,igt.id))
                    igt_skipped += 1
                    self.skipped_igt[igt.id] = 'misalignment'
                    continue
        return (igt_count, igt_skipped, igt_to_word_ids)


    def normalize_text(self,igt):
        if igt.get('m'):
            for m in igt.get('m'):
                m.text = m.value().lower().rstrip(self.ignore_chars)
                m.text = m.text.lstrip(self.ignore_chars)
        if igt.get('w'):
            for w in igt.get('w'):
                w.text = w.value().lower().rstrip(self.ignore_chars)
                w.text = w.text.lstrip(self.ignore_chars)

    def construct_pos_items(self,xc,verb_tags,noun_tags):
        words_count = 0
        words_skipped = 0
        pos_words = {}
        pos_words['verb'] = []
        pos_words['noun'] = []
        (igt_count, igt_skipped, igt_to_word_ids) = self.collect_relevant_word_ids(xc,verb_tags,noun_tags)
        for pos_tag in ['verb','noun']:
            for igt_id in igt_to_word_ids[pos_tag]:
                igt = xc.get(igt_id)
                word_to_morphs = {}
                this_igt_words = []
                success = True
                try:
                    for word_id in igt_to_word_ids[pos_tag][igt_id]:
                        word_to_morphs[word_id] = []
                        if igt.get('m'):
                            morphemes = xp.findall(igt, 'tier[@type="morphemes"]/item[referent()/@id='+word_id+']')
                            if len(morphemes) == 0:
                                if self.verbose:
                                    print('Morphemes are not aligned to a word in IGT {0}'.format(igt_id))
                                success = False
                            string = ''.join([i.value() for i in morphemes if i.value()]).strip(',.!?-=_')
                            if ''.join([i.value() for i in morphemes if i.value()]).strip(',.!?') == '':
                                if self.verbose:
                                    print('Word is the empty string or only contains punctuation'.format(igt_id))
                                    success = False
                            for m in morphemes:
                                aligned_glosses = self.find_aligned_glosses(m.id,igt)
                                if len(aligned_glosses) == 0:
                                    success = False
                                    if self.verbose:
                                        print('Morphemes are not aligned to glosses in IGT {0} word {1}'.format(igt_id, word_id))
                                    continue
                                else:
                                    word_to_morphs[word_id].append((m,aligned_glosses))
                    for wd_id in word_to_morphs:
                        if len(word_to_morphs[wd_id]) == 0:
                            words_count += 1
                            words_skipped += 1
                            if self.verbose:
                                print('w{0}, igt {1}'.format(wd_id, igt.id) + '\t' + 'No gloss')
                            self.skipped_words[igt.id][wd_id + '\t' + igt.get_item(word_id).value()] = 'No gloss'
                            continue

                        # ECC: place a check before creating a POS_Word object
                        # don't include words with "mixed morphemes" (those with affixes that are both verby and nouny)
                        # attempts to prevent mis-tagging words
                        # for example a verb w/ a nominalizer affix may be marked as a noun, even though it's a verb
                        if self.has_mixed_morphemes(pos_tag, word_to_morphs[wd_id]):
                            continue


                        pos_word = POS_Word(word_id=wd_id, igt=igt,
                                            morphemes_glosses=word_to_morphs[wd_id],
                                            patterns=self.patterns, mtg=self.mtg, hyphens=self.hyphens,
                                            lexitem_classes=self.lexitem_classes,
                                            assume_morph_boundaries = self.assume_morph_boundaries,
                                            all_bare=self.all_bare, use_gloss_for_patterns=self.use_gloss_for_patterns)
                        (text_matches, error) = pos_word.check_text(self.allowed_diff,self.allomorphs)
                        if not text_matches or pos_word.misaligned:
                            words_count += 1
                            words_skipped += 1
                            if error == 'Success':
                                error = 'misalignment'
                            if self.verbose:
                                print('{0}'.format(igt.id) + '\t' + error)
                            self.skipped_words[igt.id][wd_id + '\t' + pos_word.parent_word.value()] = error
                            continue
                        if pos_word.tokenized_text[0] in self.ungrammatical:
                            words_count += 1
                            words_skipped += 1
                        else:
                            try:
                                words = self.analyze_compound(pos_word)
                                for w in words:
                                    words_count += 1
                                    (valid_word, error) = w.validate()
                                    if not valid_word:
                                        words_skipped += 1
                                        if self.verbose:
                                            print(error + ' in word {0} in igt {1}; Skipping it'.format(wd_id,igt.id))
                                        self.skipped_words[igt.id][w.parent_word.id + '\t' + w.parent_word.value()] = error
                                    else:
                                        this_igt_words.append(w)
                            except:
                                if self.verbose:
                                    print("Possible misalignment in igt {0}".format(igt.id))
                                    error = 'misalignment'
                                    self.skipped_words[igt.id][w.parent_word.id + '\t' + w.parent_word.value()] = error
                                    raise Exception('Misalignment in igt {0}'.format(igt.id))
                    if success:
                        pos_words[pos_tag].extend(this_igt_words)
                except:
                    igt_skipped += 1
                    self.skipped_igt[igt.id] = 'misalignment'
        print('Skipped {0} out of {1} IGT. '
              'Look in output/skipped_items.txt for reason codes.'.format(igt_skipped,igt_count))
        return(words_count,words_skipped,pos_words)

    def analyze_compound(self,word):
        if not word.has_compound_tok:
            return [word]
        parts = []
        start = 0
        end = len(word.morphemes)
        for idx in word.compound_tok_indices:
            morphs = word.morphemes[start:idx]
            new_wd = POS_Word(word_id=word.original_id, igt=word.parent_igt, morphemes=morphs, mtg=self.mtg,
                              hyphens=self.hyphens,
                              lexitem_classes=self.lexitem_classes,
                              patterns=self.patterns,
                              assume_morph_boundaries = self.assume_morph_boundaries,
                              all_bare=self.all_bare, use_gloss_for_patterns=self.use_gloss_for_patterns)
            parts.append(new_wd)
            start = idx+1
        if start < end:
            morphs = word.morphemes[start:end]
            last_wd = POS_Word(word_id=word.original_id, igt=word.parent_igt, morphemes=morphs, mtg=self.mtg,
                               hyphens=self.hyphens,
                               patterns=self.patterns,
                               lexitem_classes=self.lexitem_classes,
                               assume_morph_boundaries = self.assume_morph_boundaries,
                               all_bare=self.all_bare, use_gloss_for_patterns=self.use_gloss_for_patterns)
            parts.append(last_wd)
        return parts

    def find_aligned_glosses(self,m_id, igt):
        aligned_glosses = []
        gloss_tier = igt.get('g')
        word_lvl_gloss_tier = igt.get('wt')
        if self.settings.implicitg2morph:
            g_id = 'g' + re.search('([0-9]+\.[0-9]+)',m_id).group(1)
            if igt.get_item(g_id).value() is None:
                w_id = 'w' + re.search('([0-9]+)',m_id).group(1)
                for wt in word_lvl_gloss_tier:
                    if wt.alignment == w_id:
                        aligned_glosses.append(wt)
            else:
                aligned_glosses.append(igt.get_item(g_id))
        else:
            for item in gloss_tier:
                if item.alignment == m_id:
                    aligned_glosses.append(item)
        return aligned_glosses

    def has_mixed_morphemes(self, pos_tag, morphemes):
        """
        Check to see if the list of morphemes has a mixture of 'verby' and 'nouny' morphemes

        :param pos_tag: the tag this word was already marked as
        :type pos_tag: str
        :param morphemes: the list of morphemes to check
        :type morphemes: list
        :return: whether the morphemes were mixed
        :rtype: bool
        """
        noun_glosses = FeatDict.collect_nouny_keys()
        verb_glosses = FeatDict.collect_verby_keys()

        # the word has already been marked, so set the appropriate flag
        # if the word is believed to be a noun, and we later find a verby morpheme, it should be thrown out
        if pos_tag == 'noun':
            noun_flag = True
            verb_flag = False
        elif pos_tag == 'verb':
            noun_flag = False
            verb_flag = True

        for m in morphemes:
            # split in case of a compound gloss, e.g. PST.IMP
            morph_glosses = re.split('\.|-|=', m[1][0].text.lower())

            for gloss in morph_glosses:
                if gloss in noun_glosses:
                    noun_flag = True
                elif gloss in verb_glosses:
                    verb_flag = True
                else:
                    # morpheme gloss was ambiguous, ignore it
                    continue

                if noun_flag and verb_flag:
                    # a mixture has been encountered, so the word has mixed morphemes
                    return True

        return False




class POS_Word:
    def __init__(self, word_id, igt, mtg, morphemes_glosses=None, patterns=None,
                 morphemes=None, hyphens=True, lexitem_classes=False,
                 all_bare=True, assume_morph_boundaries = True,use_gloss_for_patterns=True):
        self.original_id = word_id
        self.hyphens = hyphens
        self.misaligned = False
        self.parent_word = igt.get_item(word_id)
        self.parent_igt = igt
        self.root = None
        self.prefixes = []
        self.suffixes = []
        self.all_roots = []
        self.no_known_affix = True
        self.has_compound_tok = False
        self.compound_tok_indices = []
        self.tokenized_text = ''
        self.word_text = self.parent_word.value()
        self.is_compound = False
        self.assume_morph_boundaries = assume_morph_boundaries
        self.case_frame = {}
        self.transitivity = None
        if morphemes_glosses:
            self.morphemes = []
            i=0
            prev_tok = None
            if not assume_morph_boundaries:
                morphemes_glosses, root_indices = mtg.infer_morph_type_without_boundary(morphemes_glosses)
            for (m,g) in morphemes_glosses:
                txt = m.text if m.text else m.value()
                #TODO data-specific(? Abui) compound treatment
                tok = Word_Token(xigt_morph=m,glosses=g, patterns=patterns,real_token=True,mtg=mtg,parent_word=self,
                                 use_gloss=use_gloss_for_patterns)
                if not txt.endswith('-') and not txt.startswith('-') and '-' in txt:
                    tok.orth = re.sub('-','~~~',txt)
                self.tokenized_text += tok.orth
                if self.detect_misalignment(i=i,tok=tok, morphemes_glosses=morphemes_glosses,prev_tok=prev_tok):
                    self.misaligned = True
                    return
                if tok.type in ROOTS:
                    self.root = tok
                    self.all_roots.append(tok)
                if prev_tok:
                    #TODO data-specific(? Abui) compound treatment
                    if self.implicit_compounding(tok, prev_tok):
                        self.is_compound = True
                        ct = Word_Token(xigt_morph=None,glosses=['~'],patterns=None,
                                        real_token=False,fake_type='CMP',mtg=mtg,parent_word=self,
                                        use_gloss=use_gloss_for_patterns)
                        self.morphemes.append(ct)
                        self.has_compound_tok = True
                        self.compound_tok_indices.append(i)
                        i += 1
                    if (prev_tok.type == 'prefix' or prev_tok.type == 'proclitic') and tok.type == 'enclitic':
                        self.root = tok
                        tok.type = 'root'
                if tok.type == 'COMPOUND_TOK':
                    self.has_compound_tok = True
                    self.is_compound = True
                    self.compound_tok_indices.append(i)
                self.morphemes.append(tok)
                i += 1
                prev_tok = tok
            if not assume_morph_boundaries:
                for i in root_indices:
                    self.root = self.morphemes[i] #TODO: this only takes one root
                    self.all_roots.append(self.morphemes[i])
        else:
            if morphemes:
                self.morphemes = morphemes
                for tok in morphemes:
                    if tok.real_token:
                        self.tokenized_text += tok.orth
                    if tok.type in ROOTS:
                        self.root = tok
            #else:
            #    root_tok = Word_Token(xigt_morph=None,glosses=None, patterns=None,real_token=True,mtg=None,parent_word=self)
            #    self.morphemes = [root_tok]
            #    self.root = root_tok
            #    self.all_roots.append(root_tok)
        if self.root:
            if len(self.morphemes) > 1 and not (self.is_compound or self.has_compound_tok):
                self.ordered_morphemes = self.order_morphs(True)
            else:
                self.ordered_morphemes = list(self.morphemes)
        else:
            raise Exception("No root found in word {0} in IGT {1}".format(self.original_id, self.parent_igt.id))

        # This is currently Abui-specific. I added a "all_bare" setting
        # to choose whether to use the EMPTY- prefix or not, however,
        # the implementation does not pay attention to suffixes and so is inadequate for the name:
        # it really only shows that a stem occurs without a prefix.
        # It really should look at whether the stem is indeed bare.
        if lexitem_classes:
            if len(self.morphemes) > 1 \
                    and not self.misaligned \
                    and not (self.is_compound or self.has_compound_tok):
                try:
                    first_tok = self.ordered_morphemes[1]
                except:
                    print(self.word_text + ' ' + self.parent_igt.id)
                    raise Exception('No ordered morphemes!')
                #TODO Abui-specific(?)
                if first_tok.type == 'prefix' and first_tok.known_affix_type:
                    self.no_known_affix = False
            #TODO Abui-specific(?)
            if not all_bare:
                if self.no_known_affix and not self.misaligned and not (self.is_compound or self.has_compound_tok):
                    empty_prefix = Word_Token(xigt_morph=None,glosses=[],
                                              patterns=None,real_token=False,fake_type='DUMMY',mtg=mtg,parent_word=self,
                                              use_gloss=use_gloss_for_patterns)
                    self.morphemes.append(empty_prefix)
                    try:
                        self.ordered_morphemes.insert(self.ordered_morphemes.index(self.root)+1,empty_prefix)
                    except:
                        print(self.word_text + ' ' + self.parent_igt.id)
                        raise Exception('No ordered morphemes')

    def detect_misalignment(self,i,tok,morphemes_glosses,prev_tok=None):
        if (i == 0 and (tok.type == 'suffix' or tok.type == 'enclitic')):
            return True
        if (i == len(morphemes_glosses) - 1 and (tok.type == 'prefix' or tok.type == 'proclitic')):
            return True
        if prev_tok:
            if (prev_tok.type == 'prefix' or prev_tok.type == 'proclitic'):
                if tok.type == 'COMPOUND_TOK' or tok.type == 'suffix' or tok.type == 'enclitic':
                    return True
        return False


    def implicit_compounding(self, tok, prev_tok):
        if tok.type in ROOTS and prev_tok.type in ROOTS:
            return True
        if tok.type in ROOTS or tok.type == 'prefix' or tok.type == 'proclitic':
            if (prev_tok.type == 'suffix' or prev_tok.type == 'enclitic'):
                return True
        if (tok.type == 'prefix' or tok.type == 'proclitic'):
            if (prev_tok.type in ROOTS or prev_tok.type == 'suffix' or prev_tok.type == 'enclitic'):
                return True
        return False

    def __str__(self):
        return self.tokenized_text

    def __repr__(self):
        return self.tokenized_text

    def check_text(self, allowed_diff,allomorphs):
        reconstructed_text = re.sub('[-=\?\.\!\,\:\;\(\)\'\"\`#]','',self.tokenized_text.lower())
        phrase_text = re.sub('[-=\?\.\!\,\:\;\(\)\'\"\`#]','',self.word_text.lower())
        ed = editdistance.eval(reconstructed_text,phrase_text)
        if ed > allowed_diff \
                and not self.known_phon_diff(phrase_text,reconstructed_text,allomorphs) \
                and not self.is_compound_spelling(phrase_text,reconstructed_text):
            return(False, 'Possible misalignment: {0} aligned to {1}'.format(phrase_text,reconstructed_text))
        return (True, 'Success')

    # Is the only difference the ~ character (compound spelling)?
    def is_compound_spelling(self,str1,str2):
        diff = set(str2).difference(set(str1))
        return len(diff) == 1 and '~' in diff

    def known_phon_diff(self, str1, str2, allomorphs):
        for p in allomorphs:
            if p[2] == 0:
                if str1.startswith(p[0]) and str2.startswith(p[1]):
                    return True
            elif p[2] == -1:
                if str1.endswith(p[0]) and str2.endswith(p[1]):
                    return True
            elif p[2] == 2:
                if p[0] in str1 or p[0] in str2:
                    return True
            else:
                if str1 == p[0] and str2 == p[1]:
                    return True
        return False

    def validate(self):
        if not self.root:
            return (False, 'No root')
        for m in self.morphemes:
            if not m.orth:
                return (False, 'No orth')
            if len(m.glosses) == 0:
                return (False, 'No gloss')
        return (True, 'Success')

    def order_morphs(self, prefixes_first):
        ordered = []
        ordered.append(self.root)
        prefixes = []
        suffixes = []
        idx = self.morphemes.index(self.root)
        left_tokens = self.morphemes[0:idx]
        right_tokens = self.morphemes[idx+1:]
        for tok in left_tokens:
            if tok.real_token or tok.known_affix_type == 'empty-': #TODO: Abui-specific?
                prefixes.append(tok)
        for tok in right_tokens:
            if tok.real_token:
                suffixes.append(tok)
        prefixes.reverse()
        self.prefixes = prefixes
        self.suffixes = suffixes
        for i,p in enumerate(prefixes):
            p.position = i+1
        for i,s in enumerate(suffixes):
            s.position = i+1
        if prefixes_first:
            ordered.extend(prefixes)
            ordered.extend(suffixes)
        else:
            ordered.extend(suffixes)
            ordered.extend(prefixes)
        return ordered

class Word_Token:
    def __init__(self, xigt_morph, glosses, patterns, real_token, mtg, parent_word, fake_type=None,use_gloss=True):
        # TODO: this probably should not be part of Word_Token instances; it is confusing later
        self.known_glosses = mtg.known_glosses
        if not real_token:
            self.real_token = False
            if fake_type == 'DUMMY': #TODO: The below is Abui-specific(?)
                self.type = 'prefix'
                self.known_affix_type = 'empty-'
                self.orth = 'EMPTY-'
                self.glosses = ['EMPTY-']
                self.is_known = True
                parent_word.no_known_affix = False
            if fake_type == 'CMP':
                self.type = 'COMPOUND_TOK'
                self.known_affix_type = None
                self.glosses = ['~']
                self.orth = '~'
                self.is_known = False
            self.features = []
            self.mom_features = []
        else:
            self.type = None
            morph_text = xigt_morph.text if xigt_morph.text else xigt_morph.value()
            self.real_token = True
            if xigt_morph.type:
                self.type = xigt_morph.type.lower()
                if self.type in ROOTS:
                    self.type = 'root'
            elif parent_word.assume_morph_boundaries:
                self.type = mtg.infer_morph_type_with_boundary(morph_text)
            if self.type and self.type == 'COMPOUND_TOK':
                self.real_token = False
            self.orth = morph_text
            self.glosses = [g.value() for g in glosses]
            if parent_word.hyphens:
                insert_hyphens(self)
            self.features = []
            self.mom_features = []
            if self.type:
                self.features = self.infer_features(self.glosses)
            self.known_affix_type = self.infer_known_type(patterns,use_gloss=use_gloss)
            self.is_known = self.known_affix_type is not None
            #else:
            #    self.type = 'root'
            #    morph_text = parent_word.parent_igt.get_item(parent_word.original_id).value()
            #    gw_id = 'gw' + re.search('([0-9]+)',parent_word.original_id).group(1)
            #    self.orth = morph_text
            #    alignment = parent_word.parent_igt.get_item(gw_id).content
            #    self.glosses = [parent_word.parent_igt.get_item(gw_id)]


    def __str__(self):
        return self.orth

    def __repr__(self):
        return self.orth

    def infer_features(self, glosses):
        feature_grams = []
        for g in glosses:
            grams = re.split('\.| ',g)
            for gr in grams:
                #if gr.strip('-=') in self.known_glosses:
                if g in self.known_glosses or g.strip('-=') in self.known_glosses:
                    feature_grams.append(gr.strip('-=()+;,/').lower())#KPH removing some extra punctuation so these will match features
        return feature_grams

    def infer_known_type(self, patterns, use_gloss=True):
        precluster_strings = [self.orth] if not use_gloss else self.glosses
        if self.type in ROOTS or patterns==None:
            return None
        for p in patterns:
            for s in precluster_strings:
                match = re.search(p,s)
                if match:
                    if not p == '.+':
                        return p
                    else:
                        return s


def map_igt2words(xigt_reader):
    map = {}
    for pos in xigt_reader.items:
        for item in xigt_reader.items[pos]:
            if not item.parent_igt.id in map:
                map[item.parent_igt.id] = {}
            map[item.parent_igt.id][item.original_id] = item
    return map


def insert_hyphens(m):
    if not m.type:
        return m
    if m.type == 'prefix' and not m.orth.endswith('-'):
        m = add_hyphen(m,'end','-')
    elif m.type == 'proclitic' and not m.orth.endswith('='):
        m = add_hyphen(m,'end','=')
    elif m.type == 'suffix' and not m.orth.startswith('-'):
        m = add_hyphen(m,'start','-')
    elif m.type == 'enclitic' and not m.orth.startswith('='):
        m = add_hyphen(m,'start','=')
    return m

def add_hyphen(m,position, char):
    new_glosses = []
    if position == 'end':
        m.orth = m.orth + char
        for g in m.glosses:
            if not g.endswith(char):
                new_glosses.append(g + char)
    elif position == 'start':
        m.orth = char + m.orth
        for g in m.glosses:
            if not g.startswith(char):
                new_glosses.append(char + g)

    m.glosses = new_glosses
    return m



if __name__ == '__main__':
    xr = Xigt_Reader(sys.argv[1])



import os, sys
import re
import copy

class MorphTypeGuesser:
    def __init__(self, known_glosses_file):
        if known_glosses_file:
            with open(known_glosses_file, 'r') as f:
                glosses = [ g.strip() for g in f.readlines() ]
        else:
            glosses = set()
        self.known_glosses = set(glosses)


    def infer_morph_type_with_boundary(self, orth):
        if orth == '-' or orth == '=':
            return 'COMPOUND_TOK'
        if orth.startswith('-'):
            return 'suffix'
        if orth.endswith('-'):
            return 'prefix'
        if orth.startswith('='):
            return 'enclitic'
        if orth.endswith('='):
            return 'proclitic'
        else:
            return('root') #note: this only works for good segmentation, e.g not for Russian ODIN

    def infer_morph_type_without_boundary(self, morphemes_glosses):
        new_morphemes_glosses = []
        idx = 0
        last_root = 999
        root_indices = []
        for (m,g) in morphemes_glosses:
            new_m = copy.copy(m)
            # First see if a morpheme is clearly a gloss
            for g_i in g:
                if g_i.value() in self.known_glosses:
                    if idx < last_root:
                        new_m.type = 'prefix'
                        break
                    else:
                        new_m.type = 'suffix'
                        break
                else:
                    new_m.type = 'root'
                    last_root = idx
                    root_indices.append(idx)
                    break
            new_morphemes_glosses.append((new_m,g))
            idx += 1
        return new_morphemes_glosses, root_indices



'''
Update this dictionary with your glosses if needed.
Make sure everything you enter is consistently lower case.
'''

import sys

class IterMixin(object):
    def __iter__(self):
        for attr, value in self.__dict__.items():
            yield attr, value

'''
These dictionaries' role is:
1) to map a variety of possible glosses to a standard set
2) to map standard features to feature types (e.g. masculine to gender)
3) (possibly) to elicit mappings of unknown glosses found in the dataset from the user?
The dictionaries do not decide whether the feature should be inherent (gender on nouns)
or agreement (gender on verb's subject) feature.
'''
# Constants. These come from Leipzig glossing rules: https://www.eva.mpg.de/lingua/resources/glossing-rules.php
# Customized grammars will misbehave if two different features have the same name. To avoid this, make sure that no two
# feature categories share a feature of the same name. For example, purposive is both a mood and a case. To avoid
# conflicts in customized grammars, PURPOSIVE_MOOD and PURPOSEIVE_CASE have different values 'prp' and 'purp'
SUBJECT = 'subj' # this is cononical for head features in morphology
OBJECT = 'obj'
ZERO = '0'
FIRST = '1st' # these match the numbers used by number library
SECOND = '2nd'
THIRD = '3rd'
FOURTH = '4th'
NONFIRST = 'non-1st' # these match the numbers used by number library
NONSECOND = 'non-2nd'
NONTHIRD = 'non-3rd'
FIRSTINCL = '1i'
FIRSTEXCL = '1e'
SINGULAR = 'sg'
NONSINGULAR = 'nsg'
NONPLURAL = 'npl'
DUAL = 'du'
PLURAL = 'pl'
GREATERPAUCAL = 'gpauc'
GREATERPLURAL = 'grpl'
INVERSE = 'invn'
PAUCAL = 'pauc'
FEMININE = 'f'
MASCULINE = 'm'
NEUTER = 'n'
ANIMATE = 'anim'
INANIMATE = 'inanim'
NOMINATIVE = 'nom'
ACCUSATIVE = 'acc'
ERGATIVE = 'erg'
ABSOLUTIVE = 'abs'
GENITIVE = 'gen'
DATIVE = 'dat'
PURPOSIVE_MOOD = 'prp'
PURPOSIVE_CASE = 'purp'
PARTITIVE = 'prt'
COMPARATIVE = 'compv'
EQUATIVE = 'eqt'
PRIVATIVE = 'priv'
PROPRIETIVE = 'propr'
AVERSIVE = 'avr'
FORMAL = 'frml'
TRANSLATIVE = 'trans'
INSTRUMENTAL = 'ins'
LOCATIVE = 'loc'
INTERSETCTIVE = 'inter'
AT = 'at'
POSTERIOR = 'post'
IN = 'in'
NEAR = 'circ'
ANTERIOR = 'ante'
NEXTTO ='apud'
ON = 'on'
ONHONRIZONTAL = 'onhr'
ONVERTICAL = 'onvr'
UNDER = 'sub'
DISTAL = 'sub'
OBVIATIVE = 'obv'
PROXIMATE = 'prox'
ESSIVE = 'ess'
ALLATIVE = 'all'
ABLATIVE = 'abl'
APPROXIMATIVE = 'apprx'
TERMINATIVE = 'term'
PROLATIVE = 'prol'
VERSATIVE = 'vers'
VOCATIVE = 'voc'
OBLIQUE = 'obl'
COMITATIVE = 'com'
BENEFACTIVE = 'ben'
POSSESSIVE = 'poss'
PAST = 'pst'
DISTANT = 'dist'
PRESENT = 'pres'
ONEDAY = '1day'
FUTURE = 'fut'
NONFUTURE = 'nfut'
NONPAST = 'npst'
REMOTE = 'rem'
HODIERNAL = 'hod'
IMMEDIATE = 'immed'
RECENT = 'rct'
PRETERITE = 'pret'
SUBJUNCTIVE = 'sbjv'
NONFACTIVE = 'nfact'
INDICATIVE = 'ind'
IMPERATIVE = 'imp'
IRREALIS = 'irr'
REALIS = 'real'
OPTATIVE = 'opt'
ADMIRATIVE = 'adm'
NONPURPOSIVE = 'nprp'
CONDITIONAL = 'cond'
HYPOTHETICAL = 'hyp'
DEBITIVE = 'deb'
DEDUCTIVE = 'ded'
LIKELY = 'lkly'
OBLIGATIVE = 'oblig'
PERMISSIVE = 'perm'
PROHIBITIVE = 'proh'
SIMULATIVE = 'sim'
YES = 'plus'
IMPERFECTIVE = 'ipfv'
PERFECTIVE = 'pfv'
COMPLETIVE = 'cmpl'
INCEPTIVE = 'incep'
MOMENTANEOUS = 'mom'
RESULTATIVE = 'res'
INCLUSIVE = 'incl'
EXCLUSIVE = 'excl'
HABITUAL = 'hab'
REPETITIVE = 'rep'
ITERATIVE = 'iter'
POTENTIAL = 'pot'
USITATIVE = 'usit'
INTENTIVE = 'intt'
PROGRESSIVE = 'prog'
GRADUATIVE = 'grad'
CONTINUATIVE = 'cont'
DISTRIBUTIVE = 'distr'
PROSPECTIVE = 'prosp'
RELATIVE_ASPECT = 'rel'
DURATIVE = 'dur'
CLASSIFIER = 'cl'
CAUSATIVE = 'caus'
PASSIVE = 'pass'
ANTIPASSIVE = 'antip'
ACTIVE = 'act'
REFLEXIVE = 'refl'
RECIPROCAL = 'recip'
APPLICATIVE = 'appl'
EMPHATIC = 'emph'
ACCOMPANIERFOCUS = 'acfoc'
AGENTFOCUS = 'agfoc'
BENEFICIARYFOCUS = 'bfoc'
CONVEYEDFOCUS = 'cfoc'
INSTRUMENTFOCUS = 'ifoc'
LOCATIONFOCUS = 'lfoc'
PATIENTFOCUS = 'pfoc'
MIDDLE = 'mid'
DIMINUATIVE = 'dim'
AUGMENTATIVE = 'aug'
FINITE = 'finite'
NONFINITE = 'nonfinite'
INFINITIVE = 'inf'
NOMINALIZED = 'nmz'
TOPIC = 'top'
FOCUS = 'foc'
SUPERLATIVE = 'sprl'
ABSOLUTE = 'ab'
RELATIVE_CASE = 'rl'
DEFINITE = 'def'
INDEFINITE = 'indf'
SPECIFIC = 'spec'
NONSPECIFIC = 'nspec'
ACCOMPLISHMENT = 'accmp'
ACHEIVEMENT = 'ach'
ACTIVITY = 'acty'
ATELIC = 'atel'
DYNAMIC = 'dyn'
PUNCTUAL = 'pct'
SEMELFACTIVE = 'semel'
STATIVE = 'stat'
TELIC = 'tel'
HUMAN = 'hum'
NONHUMAN = 'nhum'
ABOVE = 'abv'
BELOW = 'bel'
EVEN = 'even'
MEDIAL = 'med'
NOREF = 'noref'
INVISIBLE = 'nvis'
PHORIC = 'phor'
FIRSTPERSONREFERENCE = 'ref1'
SECONDPERSONREFERENCE = 'ref2'
VISIBLE = 'vis'
ASSUMED = 'assum'
AUDITORY = 'aud'
DIRECT = 'drct'
FIRSTHAND = 'fh'
HEARSAY = 'hrsy'
INFERRED = 'infer'
NONFIRSTHAND = 'nfh'
NONVISUALSENSORY = 'nvsen'
QUOTATIVE = 'quot'
REPORTED = 'rprt'
SENSORY = 'sen'
DECLARATIVE = 'decl'
INTERROGATIVE = 'int'
AVOIDANCE = 'avoid'
COLLOQUIAL = 'col'
HIGHSTATUS = 'forreg'
HUMBLING = 'form'
INFORMAL = 'high'
LITERARY = 'humb'
LOWSTATUS = 'infm'
POLITE = 'lit'
ELEVATEDSTATUS = 'stelev'
SUPREMESTATUS = 'stsupr'
PARTICIPLE = 'ptcp'


subject_feature_vals = {'sbj':SUBJECT,'subject':SUBJECT,'s':SUBJECT,'subj':SUBJECT,
                        'a':SUBJECT, 's/a':SUBJECT, 'agent':SUBJECT, 'a/s':SUBJECT, 'sp':SUBJECT, 'agt':SUBJECT}

object_feature_vals = {'obj':OBJECT,'object':OBJECT,'o':OBJECT, 'p':OBJECT, 'patient':OBJECT,
                       'pat':OBJECT, 'theme':OBJECT, 'op':OBJECT}

perFeatures = {"1":FIRST, "1st":FIRST, "2nd":SECOND, "2":SECOND, "3":THIRD, "3rd":THIRD,
               "n1":NONFIRST, "non-1st":NONFIRST, "n2":NONSECOND, "non-2nd":NONSECOND,
                            "non-3rd":NONTHIRD, "n3":NONTHIRD, "4":FOURTH, "4th":FOURTH,
               "1e": FIRSTEXCL, "1i": FIRSTINCL}# removed the following becasue they are not handled by the grammar matrix , '0':ZERO, 'n0':ZERO, 'obv':OBVIATIVE, 'prx':PROXIMATE, 'prox':PROXIMATE}

numFeatures = {"sing":SINGULAR,  "singular":SINGULAR, "plural":PLURAL,
                        "dual":DUAL, "du": DUAL, "d": DUAL, "dl": DUAL,
                        "sg":SINGULAR, "pl":PLURAL, 'nons' : NONSINGULAR,
               'nonsingular' : NONSINGULAR, 'nonsing' : NONSINGULAR, 'nonsg' : NONSINGULAR,
               'nonplural': NONPLURAL, 'nonpl':NONPLURAL, 'npl': NONPLURAL, 'ns': NONSINGULAR,
               'gpauc':GREATERPAUCAL, 'grpl':GREATERPLURAL, 'invn':INVERSE, 'inv':INVERSE}

genFeatures = {"f":FEMININE, "m":MASCULINE, "n":NEUTER, 'fem':FEMININE,'masc':MASCULINE,'neut':NEUTER,
               'feminine':FEMININE,'masculine':MASCULINE,'neutral':NEUTER,'neuter':NEUTER,'animate':ANIMATE,
               'inanimate':INANIMATE,'anim':ANIMATE,'inan':INANIMATE,'inanim':INANIMATE}

perGenFeatures = {'1masc':(FIRST,MASCULINE), '2masc':(SECOND,MASCULINE), '3masc':('3rd',MASCULINE),
                               '1fem':(FIRST,FEMININE), '2fem':(SECOND,FEMININE), '3fem':('3rd',FEMININE),
                               '1neut':(FIRST,NEUTER), '2neut':(SECOND,NEUTER), '3neut':('3rd',NEUTER),
                               '1m':(FIRST,MASCULINE), '2m':(SECOND,MASCULINE), '3m':('3rd',MASCULINE),
                               '1f':(FIRST,FEMININE), '2f':(SECOND,FEMININE), '3f':('3rd',FEMININE),
                               '1n':(FIRST,NEUTER), '2n':(SECOND,NEUTER), '3n':('3rd',NEUTER)}

perNumFeatures = {"1pl":(FIRST,PLURAL), "1sg":(FIRST,SINGULAR), "2pl":(SECOND,PLURAL),
                          "2sg":(SECOND,SINGULAR), "3sg":(THIRD,SINGULAR), "3pl":(THIRD,PLURAL),
                          "1plural":(FIRST,PLURAL), "1singular":(FIRST,SINGULAR), "2plural":(SECOND,PLURAL),
                          "2singular":(SECOND,SINGULAR), "3singular":(THIRD,SINGULAR), "3plural":(THIRD,PLURAL),
                  "1du": (FIRST, DUAL), "1du": (FIRST, DUAL), "2du": (SECOND, DUAL),
                  "2du": (SECOND, DUAL), "3du": (THIRD, DUAL), "3du": (THIRD, DUAL),
                  "1dual": (FIRST, DUAL), "2dual": (SECOND, DUAL), "3dual": (THIRD, DUAL),
                  "1p": (FIRST, PLURAL), "2p": (SECOND, PLURAL), "3p": (THIRD, PLURAL),
                  "1p": (FIRST, SINGULAR), "2p": (SECOND, SINGULAR), "3p": (THIRD, SINGULAR)}

perNumGenFeatures = {'1sm':(FIRST,SINGULAR,MASCULINE),'2sm':(SECOND,SINGULAR,MASCULINE),'3sm':(THIRD,SINGULAR,MASCULINE),
                     '1pm': (FIRST, PLURAL, MASCULINE), '2pm': (SECOND, PLURAL, MASCULINE),'3pm': (THIRD, PLURAL, MASCULINE),
                     '1sf': (FIRST, SINGULAR, FEMININE), '2sf': (SECOND, SINGULAR, FEMININE),'3sf': (THIRD, SINGULAR, FEMININE),
                     '1pf': (FIRST, PLURAL, FEMININE), '2pf': (SECOND, PLURAL, FEMININE),'3pf': (THIRD, PLURAL, FEMININE),
                     '1sn':(FIRST,SINGULAR,NEUTER),'2sn':(SECOND,SINGULAR,NEUTER),'3sn':(THIRD,SINGULAR,NEUTER),
                     '1pn': (FIRST, PLURAL, NEUTER), '2pn': (SECOND, PLURAL, NEUTER),'3pn': (THIRD, PLURAL, NEUTER),
                     '2fp': (SECOND, FEMININE, OBJECT), '1fp': (FIRST, FEMININE, OBJECT),
                     '3mp': (THIRD, PLURAL, MASCULINE),
                     '2mp': (SECOND, PLURAL, MASCULINE), '1mp': (FIRST, PLURAL, MASCULINE),
                     '3np': (THIRD, PLURAL, NEUTER),
                     '2np': (SECOND, PLURAL, NEUTER), '1np': (FIRST, PLURAL, NEUTER), '3fp': (THIRD, PLURAL, FEMININE)}

# KPH I moved these to pernum because fuse and exclusive are consisidered person features in the GM
perNumInclFeatures = {'1pe': (FIRST,PLURAL,EXCLUSIVE),'1pi':(FIRST,PLURAL,INCLUSIVE),
                      '1se': (FIRST, SINGULAR, EXCLUSIVE), '1si': (FIRST, SINGULAR, INCLUSIVE)}

inclFeatures = {'incl': INCLUSIVE, 'in': INCLUSIVE, 'inc': INCLUSIVE, 'excl': EXCLUSIVE, 'ex': EXCLUSIVE, 'exc': EXCLUSIVE}


perNumInclHeadFeatures = {'1peA': (FIRST,PLURAL,EXCLUSIVE,SUBJECT),'1piA':(FIRST,PLURAL,INCLUSIVE,SUBJECT),
                          '1seA': (FIRST, SINGULAR, EXCLUSIVE, SUBJECT), '1siA': (FIRST, SINGULAR, INCLUSIVE, SUBJECT)}
# What to do with s/p?
perHeadFeatures = {"1s": (FIRST, SUBJECT), "2s": (SECOND, SUBJECT), "3s": (THIRD, SUBJECT),
                           "1s/a": (FIRST, SUBJECT), "2s/a": (SECOND, SUBJECT), "3s/a": (THIRD, SUBJECT),
                           "1a": (FIRST, SUBJECT), "2a": (SECOND, SUBJECT), "3a": (THIRD, SUBJECT),
                           #"1p": (FIRST, OBJECT), "2p": (SECOND, OBJECT), "3p": (THIRD, OBJECT), #these are plural more often than patient
                           "1s/p": (FIRST, "s/p"), "2s/p": (SECOND, "s/p"), "3s/p": (THIRD, "s/p"),
                           "1o": (FIRST, OBJECT), "2o": (SECOND, OBJECT), "3o": (THIRD, OBJECT),
                           '3I': (THIRD, OBJECT), '3II':(THIRD,OBJECT), '1subj': (FIRST, SUBJECT),
                           '2subj': (SECOND, SUBJECT), '3subj': (THIRD, SUBJECT), '1sbj': (FIRST, SUBJECT),
                           '2sbj': (SECOND, SUBJECT), '3sbj': (THIRD, SUBJECT), '1obj': (FIRST, OBJECT),
                           '2obj': (SECOND, OBJECT), '3obj': (THIRD, OBJECT)}

# Note: ns is not included because it is more commonly "nonsingular number" than "neuter gender on subject"
perGenHeadFeatures = {"1ms":(FIRST, MASCULINE, SUBJECT), "2ms":(SECOND, MASCULINE, SUBJECT),
                      "3ms":(THIRD, MASCULINE, SUBJECT),
                              "1fs":(FIRST, FEMININE, SUBJECT),
                              "2fs":(SECOND, FEMININE, SUBJECT), "3fs":(THIRD, FEMININE, SUBJECT),
                      "1mo":(FIRST, MASCULINE, OBJECT),
                              "2mo":(SECOND, MASCULINE, OBJECT), "3mo":(THIRD, MASCULINE, OBJECT),
                              "1fo":(FIRST, FEMININE, OBJECT),
                              "2fo":(SECOND, FEMININE, OBJECT), "3fo":(THIRD, FEMININE, OBJECT),
                      "1no":(FIRST, NEUTER, OBJECT),
                              "2no":(SECOND, NEUTER, OBJECT),
                              "3no":(THIRD, NEUTER, OBJECT)}

perNumHeadFeatures = {"1nsp":(FIRST,NONSINGULAR,OBJECT),"2nsp":(SECOND,NONSINGULAR,OBJECT),
                      "3nsp":(THIRD,NONSINGULAR,OBJECT),"1nss/a":(FIRST,NONSINGULAR,SUBJECT),
                      "2nss/a":(SECOND,NONSINGULAR,SUBJECT),"3nss/a":(THIRD,NONSINGULAR,SUBJECT),
                      "1sp":(FIRST,SINGULAR,OBJECT),"2sp":(SECOND,SINGULAR,OBJECT),"3sp":(THIRD,SINGULAR,OBJECT),
                      "1ss/a":(FIRST,SINGULAR,SUBJECT),"2ss/a":(SECOND,SINGULAR,SUBJECT),
                      "3ss/a":(THIRD,SINGULAR,SUBJECT), '1sa':(FIRST,SINGULAR,SUBJECT),
                      '2sa':(SECOND,SINGULAR,SUBJECT),'3sa':(THIRD,SINGULAR,SUBJECT),
                      "1pA": (FIRST,PLURAL,SUBJECT),"2pA": (SECOND,PLURAL,SUBJECT),"3pA": (THIRD,PLURAL,SUBJECT),
                      '1sA':(FIRST,SINGULAR,SUBJECT),'2sA':(SECOND,SINGULAR,SUBJECT),'3sA':(THIRD,SINGULAR,SUBJECT),
                      "1nso": (FIRST, NONSINGULAR, OBJECT), "2nso": (SECOND, NONSINGULAR, OBJECT),
                      "3nso": (THIRD, NONSINGULAR, OBJECT),"1so":(FIRST,SINGULAR,OBJECT),"2so":(SECOND,SINGULAR,OBJECT),
                      "1ss":(THIRD,SINGULAR,OBJECT), '2ss':(SECOND,SINGULAR,SUBJECT),'3ss':(THIRD,SINGULAR,SUBJECT),
                      "1pS": (FIRST,PLURAL,SUBJECT),"2pS": (SECOND,PLURAL,SUBJECT),"3pS": (THIRD,PLURAL,SUBJECT),
                      '1sS':(FIRST,SINGULAR,SUBJECT),'2sS':(SECOND,SINGULAR,SUBJECT),'3sS':(THIRD,SINGULAR,SUBJECT)}

perNumCaseFeatures = {'1sgn':(FIRST,SINGULAR,NOMINATIVE),'2sgn':(SECOND,SINGULAR,NOMINATIVE),'3sgn':(THIRD,SINGULAR,NOMINATIVE),
                      '1sgd':(FIRST,SINGULAR,DATIVE),'2sgd':(SECOND,SINGULAR,DATIVE),'3sgd':(THIRD,SINGULAR,DATIVE),
                      '1sge':(FIRST,SINGULAR,ERGATIVE),'2sge':(SECOND,SINGULAR,ERGATIVE),'3sge':(THIRD,SINGULAR,ERGATIVE),
                      '1sn': (FIRST, SINGULAR, NOMINATIVE),'2sn': (SECOND, SINGULAR, NOMINATIVE),
                      '3sn': (THIRD, SINGULAR, NOMINATIVE),'1sd': (FIRST, SINGULAR, DATIVE), '2sd': (SECOND, SINGULAR, DATIVE),
                      '3sd': (THIRD, SINGULAR, DATIVE),'1se': (FIRST, SINGULAR, ERGATIVE), '2se': (SECOND, SINGULAR, ERGATIVE),
                      '3se': (THIRD, SINGULAR, ERGATIVE),
                      '1pln':(FIRST,PLURAL,NOMINATIVE),'2pln':(SECOND,PLURAL,NOMINATIVE),'3pln':(THIRD,PLURAL,NOMINATIVE),
                      '1pld':(FIRST,PLURAL,DATIVE),'2pld':(SECOND,PLURAL,DATIVE),'3pld':(THIRD,PLURAL,DATIVE),
                      '1ple':(FIRST,PLURAL,ERGATIVE),'2ple':(SECOND,PLURAL,ERGATIVE),'3ple':(THIRD,PLURAL,ERGATIVE),
                      '1pn': (FIRST, PLURAL, NOMINATIVE),'2pn': (SECOND, PLURAL, NOMINATIVE),
                      '3pn': (THIRD, PLURAL, NOMINATIVE),'1pd': (FIRST, PLURAL, DATIVE), '2pd': (SECOND, PLURAL, DATIVE),
                      '3pd': (THIRD, PLURAL, DATIVE),'1pe': (FIRST, PLURAL, ERGATIVE), '2pe': (SECOND, PLURAL, ERGATIVE),
                      '3pe': (THIRD, PLURAL, ERGATIVE)
                      }

caseFeatures = {'nom':NOMINATIVE,'noms':NOMINATIVE,'acc':ACCUSATIVE,'erg':ERGATIVE,
                'abs':ABSOLUTIVE,'loc':LOCATIVE,'gen':GENITIVE,'instr':INSTRUMENTAL,'inst':INSTRUMENTAL,'ins':INSTRUMENTAL,
                'dat':DATIVE,'obl':OBLIQUE,'abl':ABLATIVE,'com':COMITATIVE,#'poss':POSSESSIVE,'pos':POSSESSIVE,
                'voc':VOCATIVE,'ben':BENEFACTIVE,'benef':BENEFACTIVE,'prp':PURPOSIVE_CASE,'purp':PURPOSIVE_CASE,'rel':RELATIVE_CASE,
                'prt':PARTITIVE,'compv':COMPARATIVE,'eqtv':EQUATIVE,'eqt':EQUATIVE,'priv':PRIVATIVE,'propr':PROPRIETIVE,'avr':AVERSIVE,
                'frml':FORMAL,'fml':FORMAL,'trans':TRANSLATIVE,'byway':ESSIVE,'ess':ESSIVE,'inter':INTERSETCTIVE,
                'among':INTERSETCTIVE,'at':AT,'post':POSTERIOR,'behind':POSTERIOR,'in':IN,'near':NEAR,'circ':NEAR,
                'infront':ANTERIOR,'ante':ANTERIOR,'nextto':NEXTTO,'apud':NEXTTO,'on':ON,'onhr':ONHONRIZONTAL,
                'onvr':ONVERTICAL,'sub':UNDER,'under':UNDER,'rem':DISTAL,'dist':DISTAL,'prox':PROXIMATE,'all':ALLATIVE,
                'apprx':APPROXIMATIVE,'approx':APPROXIMATIVE,'term':TERMINATIVE,'prol':PROLATIVE,'vers':VERSATIVE}

animFeatures ={'anim':ANIMATE,'hum':HUMAN,'inan':INANIMATE,'nhum':NONHUMAN}

negFeatures = {"neg":YES, "not":YES}

tenseFeatures = {"past":PAST, "pst":PAST, "pres":PRESENT, "prs":PRESENT, "fut":FUTURE,
                              "npst":NONPAST, "nfut":NONFUTURE, "future":FUTURE, "present":PRESENT,
                 'nonpast':NONPAST, 'npast':NONPAST, 'npt':NONPAST, 'nonfuture':NONFUTURE,'nonfut':NONFUTURE,'nonpst':NONPAST,'dist':DISTANT,
                 'pret':PRETERITE, 'preterite':PRETERITE, 'rem': REMOTE, 'remote':REMOTE, 'rmt':REMOTE, '1day':ONEDAY, 'oneday':ONEDAY,
                 'hod':HODIERNAL, 'hodiernal':HODIERNAL, 'immed':IMMEDIATE, 'immediate':IMMEDIATE, 'rct':RECENT,
                 'recent':RECENT}

aspectFeatures = {"imp":IMPERFECTIVE,"prf":PERFECTIVE,"impfv": IMPERFECTIVE,'imperf':IMPERFECTIVE,'ipfv':IMPERFECTIVE,
                  'impf':IMPERFECTIVE,"imperfective": IMPERFECTIVE,'impfv':IMPERFECTIVE, "pfv": PERFECTIVE, "perfective":PERFECTIVE,
                  "prfv":PERFECTIVE,"perf":PERFECTIVE,"hab": HABITUAL,'habit':HABITUAL,'prog':PROGRESSIVE,'dur':DURATIVE,'aor':PERFECTIVE,
                  'prosp':PROSPECTIVE,'prsp':PROSPECTIVE,'rel':RELATIVE_ASPECT,'ri':RELATIVE_ASPECT+'_'+IMPERFECTIVE,
                  'rp':RELATIVE_ASPECT+'_'+PERFECTIVE,'rpf':RELATIVE_ASPECT+'_'+PERFECTIVE,'distr':DISTRIBUTIVE, 'contin':CONTINUATIVE,
                  'cont':CONTINUATIVE, 'continuative':CONTINUATIVE, 'con':CONTINUATIVE, 'compl':COMPLETIVE, 'cmpl':COMPLETIVE,
                  'completive':COMPLETIVE,'comp':COMPLETIVE,'usit':USITATIVE,'usitative':USITATIVE,'inc':INCEPTIVE,
                  'incep':INCEPTIVE, 'pot':POTENTIAL, 'potential':POTENTIAL, 'rep':REPETITIVE, 'repet':REPETITIVE, 'repetitive':REPETITIVE,
                  'intentive':INTENTIVE, 'intt':INTENTIVE, 'iter':ITERATIVE, 'res':RESULTATIVE, 'result':RESULTATIVE,
                  'grad':GRADUATIVE,'graduative':GRADUATIVE,'mo':MOMENTANEOUS,'mom':MOMENTANEOUS,'momentaneous':MOMENTANEOUS}
#KPH: AOR is a weird one-- don't know what it stands for- came from ODIN

tenseAspectFeatures = {}
for tense in tenseFeatures:
    for aspect in aspectFeatures:
        tenseAspectFeatures[tense+aspect] = (tenseFeatures[tense], aspectFeatures[aspect])

# subj often is used for subjunctive but clashes with subject. What is a good way to go about it?
# Probably only looking at mood versus subject features very separately somehow?..

moodFeatures = {"sbjv":SUBJUNCTIVE, "ind":INDICATIVE,"indic":INDICATIVE,'opt':OPTATIVE,'subjunctive':SUBJUNCTIVE,'indicative':INDICATIVE,
                'optative':OPTATIVE, 'irrealis':IRREALIS,'irr':IRREALIS,'irreal':IRREALIS,'irri':IRREALIS, 'realis':REALIS,
                'real':REALIS,'imp':IMPERATIVE,'imperative':IMPERATIVE,'impv':IMPERATIVE, 'rec':RECIPROCAL, 'recip':RECIPROCAL,'recp':RECIPROCAL,
                'reciprocal':RECIPROCAL,'rr':RECIPROCAL, 'nfact':NONFACTIVE, 'nonfactive':NONFACTIVE, 'adm':ADMIRATIVE,'prp':PURPOSIVE_MOOD,
                'purp':PURPOSIVE_MOOD,'nprp':NONPURPOSIVE,'npurp':NONPURPOSIVE,'nonpurp':NONPURPOSIVE,'nonprp':NONPURPOSIVE,
                'cond':CONDITIONAL,'deb':DEBITIVE,'ded':DEDUCTIVE,'int':INTENTIVE,'inten':INTENTIVE,'lkly':LIKELY,'likely':LIKELY,
                'oblig':OBLIGATIVE,'obl':OBLIGATIVE,'opt':OPTATIVE,'perm':PERMISSIVE,'sim':SIMULATIVE, 'proh':PROHIBITIVE, 'hyp':HYPOTHETICAL}

voiceFeatures = {"appl":APPLICATIVE,'caus':CAUSATIVE,'cause':CAUSATIVE,'pass':PASSIVE,'passive':PASSIVE,'refl':REFLEXIVE,'emph':EMPHATIC,
                 'antip':ANTIPASSIVE,'antipas':ANTIPASSIVE,'antipass':ANTIPASSIVE,'anti':ANTIPASSIVE,'act':ACTIVE,'active':ACTIVE,
                 'acfoc':ACCOMPANIERFOCUS,'agfoc':AGENTFOCUS,'bfoc':BENEFICIARYFOCUS,'cfoc':CONVEYEDFOCUS,'dir':DIRECT,
                 'ifoc':INSTRUMENTFOCUS,'inv':INVERSE,'lfoc':LOCATIONFOCUS,'mid':MIDDLE,'pfoc':PATIENTFOCUS}

formFeatures = {'fin':FINITE,'nf':NONFINITE,'nonfin':NONFINITE,'nfin':NONFINITE,'inf':INFINITIVE,'nmz':NOMINALIZED,
                'nmzr':NOMINALIZED,'nmlz':NOMINALIZED,'ptcp':PARTICIPLE,'part':PARTICIPLE}

aktionsartFeatures = {'accmp':ACCOMPLISHMENT,'ach':ACHEIVEMENT,'acty':ACTIVITY,'atel':ATELIC,'dur':DURATIVE,'dyn':DYNAMIC,
                      'pct':PUNCTUAL,'semel':SEMELFACTIVE,'stat':STATIVE,'tel':TELIC}

#KPH: the classifier cat comes from ODIN and overlaps with Gender
classifierFeatures = {'cl':CLASSIFIER,'clf':CLASSIFIER,'i':CLASSIFIER,'ii':CLASSIFIER,'iii':CLASSIFIER,'iv':CLASSIFIER,
                      'v':CLASSIFIER,'vi':CLASSIFIER,'vii':CLASSIFIER,'viii':CLASSIFIER,'ix':CLASSIFIER,'x':CLASSIFIER}

sizeFeatures = {'dim':DIMINUATIVE,'dimin':DIMINUATIVE,'aug':AUGMENTATIVE}

infoStrFeatures ={'top':TOPIC,'topic':TOPIC,'foc':FOCUS,'focus':FOCUS}

compFeatures = {'cmpr':COMPARATIVE,'comp':COMPARATIVE,'sprl':SUPERLATIVE,'sup':SUPERLATIVE,'ab':ABSOLUTE,'abs':ABSOLUTIVE,
                'rl':RELATIVE_CASE,'rel':RELATIVE_CASE,'eqt':EQUATIVE}

defFeatures = {'def':DEFINITE,'indf':INDEFINITE,'spec':SPECIFIC,'nspec':NONSPECIFIC}

deixisFeatures = {'abv':ABOVE,'bel':BELOW,'even':EVEN,'med':MEDIAL,'noref':NOREF,'dist':DISTAL,'nvis':INVISIBLE,
                  'invis':INVISIBLE,'phor':PHORIC,'prox':PROXIMATE,'ref1':FIRSTPERSONREFERENCE,'ref2':SECONDPERSONREFERENCE,
                  'remt':REMOTE,'rmt':REMOTE,'vis':VISIBLE}

evidFeatures = {'assum':ASSUMED,'aud':AUDITORY,'drct':DIRECT,'dir':DIRECT,'fh':FIRSTHAND,'hrsy':HEARSAY,'infer':INFERRED,
                'nfh':NONFIRSTHAND,'nvsen':NONVISUALSENSORY,'quot':QUOTATIVE,'rprt':REPORTED,'sen':SENSORY}

interFeatures = {'decl':DECLARATIVE, 'int':INTERROGATIVE, 'q':INTERROGATIVE}

politeFeatures = {'avoid':AVOIDANCE, 'col':COLLOQUIAL, 'elev':FORMAL, 'foreg':FORMAL, 'form':FORMAL, 'high':HIGHSTATUS,
                  'humb':HUMBLING, 'hum':HUMBLING, 'infm':INFORMAL, 'lit':LITERARY, 'low':LOWSTATUS, 'pol':POLITE,
                  'stelev':ELEVATEDSTATUS, 'stsupr':SUPREMESTATUS}

possFeatures = {}


#ECC: create lists of nouny/verby keys (used in XigtReader to check for words with affixes that are both nouny and verby
nouny_feat_dicts = [caseFeatures]
verby_feat_dicts = [tenseFeatures, aspectFeatures, moodFeatures, voiceFeatures, formFeatures]


def collect_nouny_keys():
    return set().union(*nouny_feat_dicts)


def collect_verby_keys():
    return set().union(*verby_feat_dicts)


class FeatureDictionary(IterMixin):

    def __init__(self):
        self.dictionaries = {'aktionsartFeatures':aktionsartFeatures,
                             'animFeatures':animFeatures,
                             'aspectFeatures': aspectFeatures,
                             'caseFeatures': caseFeatures,
                             'classifierFeatures': classifierFeatures,
                             'compFeatures': compFeatures,
                             'defFeatures': defFeatures,
                             'deixisFeatures': deixisFeatures,
                             'evidFeatures': evidFeatures,
                             'formFeatures': formFeatures,
                             'genFeatures': genFeatures,
                             'infoStrFeatures': infoStrFeatures,
                             'interFeatures': interFeatures,
                             'moodFeatures': moodFeatures,
                             'negFeatures': negFeatures,
                             'numFeatures': numFeatures,
                             'object_feature_vals': object_feature_vals,
                             'perFeatures': perFeatures,
                             'perGenFeatures': perGenFeatures,
                             'perHeadFeatures': perHeadFeatures,
                             'perGenHeadFeatures': perGenHeadFeatures,
                             'perNumFeatures': perNumFeatures,
                             'perNumCaseFeatures': perNumCaseFeatures,
                             'perNumGenFeatures': perNumGenFeatures,
                             'perNumHeadFeatures': perNumHeadFeatures,
                             'perNumInclFeatures': perNumInclFeatures,
                             'perNumInclHeadFeatures': perNumInclHeadFeatures,
                             'politeFeatures': politeFeatures,
                             'sizeFeatures': sizeFeatures,
                             'subject_feature_vals': subject_feature_vals,
                             'tenseFeatures': tenseFeatures,
                             'tenseAspectFeatures': tenseAspectFeatures,
                             'voiceFeatures': voiceFeatures
                             }
        self.all_keys = self.collect_all_keys()

        #ECC: nouny and verby key lists
        # self.nouny_keys = collect_nouny_keys()
        # self.verby_keys = collect_verby_keys()

        # Variable names used in the Grammar Matrix. Cannot use any of these since then the Grammar Matrix will
        # not be able to properly compile the grammar.

        self.internal_feature_name = ["p", "i", "a", "x", "e", "tense", "sf", "comp", "cat", "conj", "rel", "agr",
                                              "opt", "cont", "sign", "number", "adv", "topic", "complete", "det",
                                              "scratch", "word", "passive", "last", "mod", "head", "per",
                                              "activated", "altkey", "altkeyrel", "alts", "apparg", "arg",
                                              "args", "arg-st", "aspect", "body", "carg", "c-arg", "c-cont",
                                              "cform", "clause", "clause-key", "cog-st", "compkey", "comps",
                                              "conj-dtr", "coord", "coord-rel", "coord-strat", "ctxt", "dtr", "fc",
                                              "first", "gtop", "harg", "hc-light", "hcons", "head-dtr", "hook", "icons",
                                              "icons-key", "index", "inflected", "instloc", "key", "key-arg", "keycomp",
                                              "keyrel", "keys", "label-name", "larg", "lbl", "lcoord-dtr", "l-hndl", "light",
                                              "l-index", "list", "lkeys", "local", "l-periph", "ltop", "marker-dtr", "mc",
                                              "meta-prefix", "meta-suffix", "mkg", "modified", "mood", "needs-affix", "node",
                                              "nonconj-dtr", "non-head-dtr", "non-local", "non-marker-dtr", "ocompkey",
                                              "opt-cs", "periph", "png", "posthead", "pred", "presup", "que", "rcoord-dtr",
                                              "rels", "rest", "result", "r-hndl", "r-index", "r-periph", "rstr", "rule-name",
                                              "slash", "sort", "spec", "speci", "spr", "stem", "subj", "synsem", "target",
                                              "tp", "val", "wlink", "xarg"]

    def collect_all_keys(self):
        all_keys = set()
        for d in self.dictionaries:
            for k in self.dictionaries[d]:
                all_keys.add(k)
        return all_keys

    def validate_dictionary(self):
        for d in self.dictionaries:
            for key, val in self.dictionaries[d].items():
                if self.lowercase_error(key) or self.lowercase_error(val):
                    sys.exit(1)


    def lowercase_error(self,item):
        for ch in item:
            if not (ch.isdigit() or ch.islower()):
                print('Found a non-lowercase item in FeatDict.py; '
                      'please make sure everything there is lowercase: ' + item)
                return True
            else:
                return False



import sys
from xigt.codecs import xigtxml


xigt_file = sys.argv[1]
word = sys.argv[2]

pos_tags = set()
glosses = set()

with open(xigt_file, encoding='utf-8') as file:
    xc = xigtxml.load(file)

for igt in xc:
    if igt.get('m'):
        for item in igt.get('m'):
            if item.value() == word:
